/**************************************************************************************************
* NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
*
* Copyright 2010 Flexera Software, Inc.  All Rights Reserved.  
*
* This computer program is the property of Flexera Software, Inc.  Any use, copy, publication, 
* distribution, display, modification, or transmission of this computer program in whole or in part
* in any form or by any means without the prior express written permission of Flexera Software, 
* Inc. is strictly prohibited.
* 
* Except when expressly provided by Flexera Software, Inc. in writing, possession of this computer 
* program shall not be construed to confer any license or rights under any of Flexera Software,
* Inc.'s intellectual property rights, whether by estoppel, implication, or otherwise.  
* 
* ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
**************************************************************************************************/
/*
	This source file is part of a sample utility that demonstrates Composite Transactions.

	It defines the program arguments.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "version_info.h"
#include "version_hotfix.h"

#include "compTranUtilCommandDef.h"
#include "compTranUtilCommand.h"
#include "compTranUtilContext.h"

#include "FlxActCommon.h"

static FlxCtuCommandDef* sGetCommandDef(const char*pszName);
static void sDefineAttributes();
static void sDefineAttribute(FlxCtuCommandDef*	pCommandDef, 
							 const char*		pszName, 
							 const char*		pszAlias,
							 ...);				/* Allowed value strings terminated by NULL. */
static void sFreeAttributesDefs(FlxCtuCommandDef* pCommandDef);
static void sFreeAttributesDef(FlxCtuAttributeDef* pAttributeDef);

static FlxCtuCommandDef sCommandDefs[] =
{
	{ CMD_ACTIVATE,			"a", flxCtuCommandTraitRequestActionActivate, 0, NULL },
	{ CMD_BRIEF,			"b", flxCtuCommandTraitNone, 0, NULL },
	{ CMD_CANCEL_REQUEST,	"c", flxCtuCommandTraitCommandCancel, 0, NULL },
	{ CMD_REPAIR,			"e", flxCtuCommandTraitRequestActionRepair, 0, NULL },
	{ CMD_LOG_ERRORS,		"g", flxCtuCommandTraitNone, 0, NULL },
	{ CMD_HELP,				"h", flxCtuCommandTraitCommandHelp, 0, NULL },
	{ CMD_LIST_REQUEST,		"l", flxCtuCommandTraitCommandList, 0, NULL },
	{ CMD_NEW,				"n", flxCtuCommandTraitCanHaveValue, 0, NULL },
	{ CMD_VERBOSE,			"o", flxCtuCommandTraitIsVerbose, 0, NULL },
	{ CMD_PROCESS,			"p", flxCtuCommandTraitCommandProcess, 0, NULL },
	{ CMD_RETURN,			"r", flxCtuCommandTraitRequestActionReturn, 0, NULL },
	{ CMD_STORED,			"s", flxCtuCommandTraitCanHaveValue, 0, NULL },
	{ CMD_TRANSACTION,		"t", flxCtuCommandTraitCommandTransaction},
	{ CMD_VIEW,				"v", flxCtuCommandTraitCommandView},

	{ CMD_CANCEL_ALL,		"",  flxCtuCommandTraitNone, 0, NULL },
	{ CMD_DELETE_FR,		"",  flxCtuCommandTraitCommandDelete, 0, NULL },
	{ CMD_REPAIR_ALL,		"",  flxCtuCommandTraitIsRequestAction, 0, NULL },
};
static size_t sCommandDefsCount = sizeof(sCommandDefs) / sizeof(sCommandDefs[0]);

static FlxActBool sbIsAttributeDefsDefined = FLX_ACT_FALSE;

size_t flxCtuCommandDefsCreate(const FlxCtuCommandDef* ppCommandDefs[] )
{
	if (!sbIsAttributeDefsDefined)
	{
		sDefineAttributes();
		sbIsAttributeDefsDefined = FLX_ACT_TRUE;
	}
	*ppCommandDefs = sCommandDefs;
	return sCommandDefsCount;
}

void flxCtuCommandDefsDestroy(FlxCtuCommandDef pCommandDefs[], size_t count)
{
	FLX_CTU_ASSERT(pCommandDefs == sCommandDefs);
	FLX_CTU_ASSERT(count == sCommandDefsCount);

	if (sbIsAttributeDefsDefined)
	{
		size_t index;
		for (index = 0; index < sCommandDefsCount; index++)
			sFreeAttributesDefs(&pCommandDefs[index]);
		
		sbIsAttributeDefsDefined = FLX_ACT_FALSE;
	}
}

static void sHelpBrief(const FlxCtuContext* pContext);
static void sHelpCancel(const FlxCtuContext* pContext);
static void sHelpDeleteFr(const FlxCtuContext* pContext);
static void sHelpList(const FlxCtuContext* pContext);
static void sHelpLogErrors(const FlxCtuContext* pContext);
static void sHelpNew(const FlxCtuContext* pContext);
static void sHelpProcess(const FlxCtuContext* pContext);
static void sHelpRepairAll(const FlxCtuContext* pContext);
static void sHelpStored(const FlxCtuContext* pContext);
static void sHelpSummary(const FlxCtuContext* pContext);
static void sHelpTransaction(const FlxCtuContext* pContext);
static void sHelpView(const FlxCtuContext* pContext);
static void sHelpVerbose(const FlxCtuContext* pContext);

/***************************************************************************************************
*	flxCtuCommandHelp
*
* -help							Summary of commands and topics
* -help <command or topic>		e.g "-help new"
* -help all						All help as continuous output.
***************************************************************************************************/
void flxCtuCommandHelp(const FlxCtuContext* pContext, const FlxCtuCommand* pCommand)
{
	const char *pszHelpContext = NULL;
	FlxActBool bShowAll;

	flxCtuPrintMessage("Composite Transaction Utility %u.%u.%u.%u\n", FNP_VER_MAJOR,
																	  FNP_VER_MINOR,
																	  FNP_VER_MAINT,
																	  FNP_VER_HOTFIX);

	if ( (pCommand == NULL) ||!flxCtuCommandHasValue(pCommand) )
	{
		sHelpSummary(pContext);
		return;
	}

	pszHelpContext  = flxCtuCommandGetValue(pCommand);
	bShowAll		= (0 == flxCtuStrCmpLwr(pszHelpContext, "all"))? FLX_ACT_TRUE : FLX_ACT_FALSE;

	if (bShowAll)
	{
		sHelpSummary(pContext);
		sHelpTransaction(pContext);
		sHelpNew(pContext);
		sHelpStored(pContext);
		sHelpProcess(pContext);
		sHelpList(pContext);
		sHelpCancel(pContext);
		sHelpView(pContext);
		sHelpDeleteFr(pContext);
		sHelpRepairAll(pContext);
		sHelpBrief(pContext);
		sHelpLogErrors(pContext);
		sHelpVerbose(pContext);
	}
	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_TRANSACTION))
		sHelpTransaction(pContext);

	else if (	(0 == flxCtuStrCmpLwr(pszHelpContext, CMD_NEW))
			 || (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_ACTIVATE))
			 || (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_RETURN))
			 || (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_REPAIR)) ) 
		sHelpNew(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_STORED))
		sHelpStored(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_PROCESS))
		sHelpProcess(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_LIST_REQUEST))
		sHelpList(pContext);

	else if (	(0 == flxCtuStrCmpLwr(pszHelpContext, CMD_CANCEL_REQUEST))
			 || (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_CANCEL_ALL)) )
		sHelpCancel(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_VIEW))
		sHelpView(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_DELETE_FR))
		sHelpDeleteFr(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_REPAIR_ALL))
		sHelpRepairAll(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_BRIEF))
		sHelpBrief(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_LOG_ERRORS))
		sHelpLogErrors(pContext);

	else if (0 == flxCtuStrCmpLwr(pszHelpContext, CMD_VERBOSE))
		sHelpVerbose(pContext);

	else
	{
		flxCtuPrintMessage("\nNo specific help available for \"%s\"\n\n", pszHelpContext);
		sHelpSummary(pContext);
	}
}

/***************************************************************************************************
*	sDefineAttributes
*
* Adds the attribute definitions to the command definitions.
***************************************************************************************************/
void sDefineAttributes()
{
	FlxCtuCommandDef* pCommandDef;

	/* Attributes for CMD_ACTIVATE */
	pCommandDef = sGetCommandDef(CMD_ACTIVATE);
	sDefineAttribute(pCommandDef, CMD_ATTR_COUNT,	 "", ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_EXP_TYPE, "exptype", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_EXP_VALUE,"exp", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_FID,		 "fid", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_FR,		 "fr", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_REASON,	 "", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_RID_TYPE, "type", NULL);

	sDefineAttribute(pCommandDef, CMD_ATTR_ACTIVATABLE,   "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_ACTIVATABLE_OD,"activatableo", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_CONCURRENT,    "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_CONCURRENT_OD,  "concurrento", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_HYBRID,		  "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_HYBRID_OD,	  "hybrido", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_REPAIR_COUNT,  "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);

	/* Attributes for CMD_DELETE_FR */
	pCommandDef = sGetCommandDef(CMD_DELETE_FR);
	sDefineAttribute(pCommandDef, CMD_ATTR_FEATURE, "", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_ENABLED, "", ATTR_VALUE_YES, ATTR_VALUE_NO, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_PID,		"pid", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_TRUSTED, "", ATTR_VALUE_YES, ATTR_VALUE_NO, NULL);

	/* Attributes for CMD_LIST_REQUEST */
	pCommandDef = sGetCommandDef(CMD_LIST_REQUEST);
	sDefineAttribute(pCommandDef, CMD_ATTR_FORMAT,	  "", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, 
															 ATTR_VALUE_XML, NULL);

	/* Attributes for CMD_NEW */
	pCommandDef = sGetCommandDef(CMD_NEW);
	sDefineAttribute(pCommandDef, CMD_ATTR_REFERENCE, "ref", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_FR,		  "fr", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, 
															ATTR_VALUE_NONE, NULL);

	/* Attributes for CMD_REPAIR */
	pCommandDef = sGetCommandDef(CMD_REPAIR);
	sDefineAttribute(pCommandDef, CMD_ATTR_FR, "fr", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, NULL);

	/* Attributes for CMD_RETURN */
	pCommandDef = sGetCommandDef(CMD_RETURN);
	sDefineAttribute(pCommandDef, CMD_ATTR_COUNT,	"", ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_FR,		"fr", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_REASON,	"", NULL);

	sDefineAttribute(pCommandDef, CMD_ATTR_ACTIVATABLE,   "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_ACTIVATABLE_OD,"activatableo", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_CONCURRENT,    "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_CONCURRENT_OD,  "concurrento", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_HYBRID,		  "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_HYBRID_OD,	  "hybrido", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_REPAIR_COUNT,  "", 
														  ATTR_VALUE_ALL, ATTR_VALUE_NUMERIC, NULL);

	/* Attributes for CMD_STORED */
	pCommandDef = sGetCommandDef(CMD_STORED);
	sDefineAttribute(pCommandDef, CMD_ATTR_REQUEST, "", "#", NULL);

	/* Attributes for CMD_TRANSACTION */
	pCommandDef = sGetCommandDef(CMD_TRANSACTION);
	sDefineAttribute(pCommandDef, CMD_ATTR_POLL,	"poll", "#", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_PROXY,	"", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_SSL,		"", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_TIMEOUT, "time", "#", NULL);

	/* Attributes for CMD_VIEW */
	pCommandDef = sGetCommandDef(CMD_VIEW);
	sDefineAttribute(pCommandDef, CMD_ATTR_FEATURE, "", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_ENABLED, "", ATTR_VALUE_YES, ATTR_VALUE_NO, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_FORMAT,	"", ATTR_VALUE_SHORT, ATTR_VALUE_LONG, NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_PID,		"pid", NULL);
	sDefineAttribute(pCommandDef, CMD_ATTR_TRUSTED, "", ATTR_VALUE_YES, ATTR_VALUE_NO, NULL);
}

/***************************************************************************************************
*	sDefineAttribute
*
* Adds an attribute definition to a command definition.
***************************************************************************************************/
static void sDefineAttribute(FlxCtuCommandDef*	pCommandDef, 
							 const char*		pszName, 
							 const char*		pszAlias,
							 ...)				/* Allowed value strings terminated by NULL. */
{
	const char*			pszValue	  = NULL;
	FlxCtuAttributeDef* pAttributeDef = NULL;
	va_list				argList;

	/* Create a new attribute definition. */
	pAttributeDef = (FlxCtuAttributeDef*)flxCtuMallocAndZeroise(sizeof(FlxCtuAttributeDef));
	pAttributeDef->pszName  = pszName;
	pAttributeDef->pszAlias = pszAlias;

	/* Add the allowed values from the variable argument list, which is terminated by NULL. */
	va_start(argList, pszAlias);
	while (NULL != (pszValue = va_arg(argList, const char*)))
	{
		/* Extend the list. */
		size_t newSize = (pAttributeDef->valueCount + 1) * sizeof(const char*);
		pAttributeDef->ppszValues = (const char**)flxCtuRealloc(pAttributeDef->ppszValues, 
																newSize);
		/* Set the allowed value as the last element. */
		pAttributeDef->ppszValues[pAttributeDef->valueCount++] = pszValue;
	}
	va_end(argList);

	/* Append the attribute definition to the list in the command definition. */
	{
		/* Extend the list. */
		size_t newSize = (pCommandDef->attributeDefCount + 1) * sizeof(FlxCtuAttributeDef*);
		pCommandDef->ppAttributeDefs = (FlxCtuAttributeDef**)
									   flxCtuRealloc(pCommandDef->ppAttributeDefs, newSize);
		/* Set the definition as the last element. */
		pCommandDef->ppAttributeDefs[pCommandDef->attributeDefCount++] = pAttributeDef;
	}


}

/***************************************************************************************************
*	sFreeAttributesDef
*
* Frees an attribute definitions.
***************************************************************************************************/
static void sFreeAttributesDef(FlxCtuAttributeDef* pAttributeDef)
{
	/* Free the value list. */
	flxCtuFree(pAttributeDef->ppszValues);

	/* Free the definition itself. */
	flxCtuFree(pAttributeDef);
}

/***************************************************************************************************
*	sFreeAttributesDefs
*
* Frees the attribute definitions of a command definition.
***************************************************************************************************/
static void sFreeAttributesDefs(FlxCtuCommandDef* pCommandDef)
{
	if (pCommandDef->attributeDefCount > 0)
	{
		size_t attrDefIndex;

		/* Free each attribute definition. */
		for (attrDefIndex = 0; attrDefIndex < pCommandDef->attributeDefCount; attrDefIndex++)
			sFreeAttributesDef(pCommandDef->ppAttributeDefs[attrDefIndex]);

		/* Free the list. */
		flxCtuFree(pCommandDef->ppAttributeDefs);
	}	
}

/***************************************************************************************************
*	sGetCommandDef
*
* Return the command definition for a given command name.
***************************************************************************************************/
static FlxCtuCommandDef* sGetCommandDef(const char*pszName)
{
	size_t index;

	for (index = 0; index < sCommandDefsCount; index++)
	{
		if (0 == strcmp(sCommandDefs[index].pszName, pszName))
			return &sCommandDefs[index];
	}
	FLX_CTU_ASSERT(FLX_ACT_FALSE);
	return NULL;
}

/***************************************************************************************************
*	sHelpBrief
*
* Show the usage of the "brief" option.
***************************************************************************************************/
static void sHelpBrief(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"brief option: print output and error messages only.\n"
		"=====\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpCancel
*
* Show the command line usage for the -cancel and -cancelAll commands.
***************************************************************************************************/
static void sHelpCancel(const FlxCtuContext* pContext)
{
	flxCtuPrintMessage(
		"\n"
		"cancelRequest command: cancel a stored composite request because no response\n"
		"=============          is expected.\n"
		"\n"
		"      -cancelRequest|-c <sequenceNo>]\n"	
		"\n"
		"  Example - cancel the composite request with sequence number 23:\n"
		"      -cancelRequest 23 \n"
		"\n"
		"cancelAll command: cancel all stored composite requests.\n"
		"=========\n"
		"      -cancelAll\n"	
		);
	return;
}

/***************************************************************************************************
*	sHelpDeleteFr
*
* Show the command line usage for the -delete command.
***************************************************************************************************/
static void sHelpDeleteFr(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(
		"\n"
		"deleteFR command: delete fulfillment record(s) from trusted storage.\n"
		"========\n"
		"    -deleteFR [<fulfillmentid>] [<filters>]\n"	
		"\n"
		"  Filters:\n"
		"    fulfillmentId        The record with this id.\n"
		"    productId|pid=<pid>  Only records with this ProductId.\n"
		"    feature=<string>     Only records with <string> in the feature line.\n"
		"    trusted=yes|no       Only trusted|untrusted records.\n"
		"    enabled=yes|no       Only enabled|disabled records.\n"
		"\n"
		"  The fulfillments to be deleted are displayed and confirmation requested\n"
		"  (use -brief to suppress confirmation).\n"
		"\n"
		"  Example - delete single fulfillment record\n"
		"      -delete MyFulfillmentId\n"
		"\n"
		"  Example - delete fulfillment records for \"MyProduct\"\n"
		"      -delete productId=MyProduct\n"
		"\n"
		"  Example - delete untrusted fulfillment records\n"
		"      -delete trusted=no\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpList
*
* Show the command line usage for the listRequests command.
***************************************************************************************************/
static void sHelpList(const FlxCtuContext* pContext)
{
	flxCtuPrintMessage(
		"\n"
		"listRequests command: list the composite request(s) that are stored\n"
		"============          awaiting a response.\n"
		"\n"
		"      -listRequests|-l [<seqNo>] [format=<formatType>]\n"
		"\n"
		"  seqNo       The request with this sequence number, default all\n"
		"\n"
		"  Format types (default is \"short\"):\n"
		"    short     <seqNo> <status> <time created> <reference>\n"
		"    long      Multi-line with format <name>=<value>\n"
		"    xml       The full xml of the request(s)\n"
		"\n"
		"  Example - list all requests in default format:\n"
		"      -l \n"
		"\n"
		"  Example - list all requests, multi-line:\n"
		"      -l format=long\n"
		"\n"
		"  Example - list request with sequence number 23, as XML:\n"
		"      -l 23 format=xml\n"
		);
}

/***************************************************************************************************
*	sHelpLogErrors
*
* Show the usage of the "logErrors" option.
***************************************************************************************************/
static void sHelpLogErrors(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"logErrors option: write API error details to the FNP log file.\n"
		"=========\n"
		);
}

/***************************************************************************************************
*		sHelpNew
*
*	Show the usage for the "new" command.
***************************************************************************************************/
static void sHelpNew(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(
		"\n"
		"new command: create a new composite request.\n"
		"===\n"
		"  -new|-n [<filename>] [fulfillmentRecord|fr=short|long|none]\n"
		"          [reference|ref=<ref>] [<gDictionary>...] [<action> ...]\n"
		"\n"
		"  <filename>   Request is written to this file; optional, default stdout.\n"
		"  <ref>        A reference that can be used to identify the request later.\n"
		"\n"
		"  Global dictionaries <gDictionary>:\n"
		"    gFLEXnet|gVendor{[<key1>=<value1>] [<key2>=<value2>]..., NULL }\n"
		"\n"
		"  Actions:\n"
		"   -activate|-a <RightsId> [<attribute>=<value> ...] [<dictionary>...]\n"
		"     attributes: count rightsType|type reason\n"
		"                 expiration|exp expirationType|expType\n"
		"                 fulfillmentId|fid fulfillmentRecord|fr\n"
		"                 activatable activatableOd concurrent concurrentOd\n"
		"                 hybrid hybridOd repairCount\n"
		"   -return|-r <FulfillmentId> [<attribute>=<value> ...] [<dictionary>...]\n"
		"     attributes: count reason fulfillmentRecord|fr\n"
		"                 activatable activatableOd concurrent concurrentOd\n"
		"                 hybrid hybridOd repairCount\n"
		"   -repair|-e <FulfillmentId> [<dictionary>...]\n"
		"     attributes: fulfillmentRecord|fr\n"
		"   -repairAll\n"
		"\n"
		"  Action dictionaries <dictionary>:\n"
		"    FLEXnet|Vendor{[<key1>=<value1>] [<key2>=<value2>]..., NULL }\n"
		"\n"
		"  Example 1 - activation using rights \"MyRights\":\n"
		"      -new request1.xml reference=\"Example 1\" -activate MyRights\n"
		"\n"
		"  Example 2 - return all of fulfillment \"FF1\" and part (5) of \"FF2\":\n"
		"      -n request2.xml -r FF1 -r FF2 count=5\n"
		, pContext->pszProgramName, pContext->pszProgramName);
}

/***************************************************************************************************
*	sHelpProcess
*
* Show the usage of the "process" command.
***************************************************************************************************/
static void sHelpProcess(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"process command: process a composite response.\n"
		"=======\n"
		"      -process|-p <filename>\n"
		"\n"
		"  <filename> Response is read from this file.\n"
		"\n"
		"  Example - process response from file response.xml:\n"
		"      -p response.xml\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpRepairAll
*
* Show the usage of the "repairAll" command.
***************************************************************************************************/
static void sHelpRepairAll(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"repairAll command: create a request containing a repair action for\n"
		"=========          every fulfillment record that is not fully trusted.\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpStored
*
* Show the usage of the "stored" command.
***************************************************************************************************/
static void sHelpStored(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"stored command: retrieve a previously created request from Trusted Storage.\n"
		"======\n"
		"      -stored|-s <filename> [request=<sequenceNo>]\n"
		"\n"
		"  <filename>   Request is written to this file; default stdout.\n"
		"  <sequenceNo> The sequence number of the request; default is most recent.\n"
		"\n"
		"  Example 1 - write the most recent request to \"latestRequest.txt\":\n"
		"       -stored latestRequest.txt\n"
		"\n"
		"  Example 2 - write stored request 23 to stdout:\n"
		"       -s request=23\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpSummary
*
* Show a summary of the usage, and how to get more details.
***************************************************************************************************/
static void sHelpSummary(const FlxCtuContext* pContext)
{
	const char* pszProgramName = pContext->pszProgramName;

	flxCtuPrintOutput(	
		"\n"
		"Commands:\n"
		" -n|-new [<filename>] [<attribute>...] [<dictionary>...] [<action>...]\n"
		" -s|-stored [<filename>] [request=<SequenceNo>]\n"
		" -t|-transaction <ServerURI> [<attribute>...] <new>|<stored>\n"
		" -p|-process <filename>\n"
		" -v|-view [<FulfillentId>] [<filter>] [<format>]\n"
		"    -deleteFR [<FulfillentId>] [confirm=yes|no] [<filter>]\n"
		" -l|-listRequests [<SequenceNo>] [<filter>] [<format>]\n"
		" -c|-cancelRequest <SequenceNo>\n"
		"    -cancelAll\n"
		"\n"
		"Actions:\n"
		" -a|-activate <RightsId> [<attribute>...] [<dictionary>...]\n"
		" -r|-return <FulfillentId> [<attribute>...] [<dictionary>...]\n"
		" -e|-repair <FulfillentId> [<attribute>...] [<dictionary>...]\n"
		"\n"
		"Options:\n"
		" -b|-brief\n"
		" -g|-logErrors\n"
		" -o|-verbose\n"
		"\n"
#if 0
		"Other help topics:\n"
		"    action          dictionary      syntax\n"
		"\n"
#endif
		"For help on a command or option use -help <name> e.g.\n"
		"    %s -help new\n"
		"\n"
		"For all commands and options:\n"
		"    %s -help all\n"
		"\n", pszProgramName, pszProgramName, pszProgramName);
	return;
}

/***************************************************************************************************
*	sHelpTransaction
*
* Show the usage of the "transaction" command.
***************************************************************************************************/
static void sHelpTransaction(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"transaction command: perform a composite transaction with a server.\n"
		"===========\n"
		"      -transaction|-t <ServerURI> [<attribute>...] <new>|<stored>\n"
		"\n"
		"  Attributes (all optional):\n"
		"    proxy = \"<host> <port> [<userId> <passwd>]\"\n"
		"    ssl = \"<CaCert> <CaPath>\"\n"
		"    timeout|time = <seconds>\n"
		"    pollInterval|poll = <seconds>\n"
		"  Either a new request or a previously created (stored) one may be specified.\n"
		"\n"
		"  Example - create a new request, send to TheServer and process the response:\n"
		"       -t http://TheServer:7777/serviceDetails -new MyRightsId\n"
		"\n"
		"  Example - use the most recently created request:\n"
		"       -t http://TheServer:7777/serviceDetails -stored\n"
		"\n"
		"  Example - use stored request with sequence number 23:\n"
		"       -t http://TheServer:7777/serviceDetails -s request=23\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpView
*
* Show the command line usage for the -view command.
***************************************************************************************************/
static void sHelpView(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(
		"\n"
		"view command: view the fulfillment records in trusted storage, including those\n"
		"====          created by means other than composite transactions.\n"
		"\n"
		"    -view|-v [<fulfillmentid>] [<filters>] [format=<formatType>]\n"	
		"\n"
		"  Filters:\n"
		"    fulfillmentId        The record with this Fulfillment Id.\n"
		"    productId|pid=<pid>  Only records with this Product Id.\n"
		"    feature=<string>     Only records with <string> in the feature line.\n"
		"    trusted=yes|no       Only trusted|untrusted records.\n"
		"    enabled=yes|no       Only enabled|disabled records.\n"
		"\n"
		"  Format types (default is short):\n"
		"    short     U D FulfillmentType Expiration Count FulfillmentId ProductId\n"
		"              where U is replaced by . if trusted,  D by . if enabled.\n"
		"    long      Multi-line with format <name>=<value>\n"
		"\n"
		"  Example - all records in default format:\n"
		"      -v \n"
		"\n"
		"  Example - records for \"MyProduct\" with feature \"ADD\":\n"
		"      -v product=MyProduct feature=ADD\n"
		"\n"
		"  Example - record with fulfillment id \"FID_123\", multi-line:\n"
		"      -v FID-123 format=long\n"
		"\n"
		"  Example - disabled records:\n"
		"      -v enabled=no\n"
		);
	return;
}

/***************************************************************************************************
*	sHelpVerbose
*
* Show the usage of the "verbose" option.
***************************************************************************************************/
static void sHelpVerbose(const FlxCtuContext* pContext)
{
	flxCtuPrintOutput(	
		"\n"
		"verbose option: print additional information about actions and errors.\n"
		"=======\n"
		);
	return;
}

